import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';
import 'package:code_builder/code_builder.dart';
import 'package:dartrofit/dartrofit.dart';
import 'package:dartrofit_generator/src/webapi_source_class.dart';
import 'package:source_gen/source_gen.dart';
import 'package:dart_style/dart_style.dart';

class WebApiGenerator extends GeneratorForAnnotation<WebApi> {
  @override
  String generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    return WebApiSourceClass(element as ClassElement).generatedCode();
  }
}

class BaseUrlGenerator extends Generator {
  final Map<String, dynamic> config;

  BaseUrlGenerator(this.config);

  @override
  String generate(LibraryReader library, BuildStep buildStep) {
    final baseUrlTypeCls = Class((b) => b
      ..name = 'BaseUrlType'
      ..fields.addAll(config.keys.map((baseUrlType) {
        return Field((b) => b
          ..static = true
          ..modifier = FieldModifier.constant
          ..type = refer('String')
          ..name = baseUrlType
          ..assignment = Code('\'$baseUrlType\''));
      })));

    final envUrl = config.values.first as Map;
    final webEnvCls = Class((b) => b
      ..name = 'WebEnv'
      ..fields.addAll(envUrl.keys.map((env) {
        return Field((b) => b
          ..static = true
          ..modifier = FieldModifier.constant
          ..type = refer('String')
          ..name = env.toString()
          ..assignment = Code('\'$env\''));
      })));

    final instanceMethod = Method.returnsVoid((b) => b
      ..name = 'autoConfigureBaseUrl'
      ..requiredParameters.add(Parameter((b) => b
        ..name = 'mgr'
        ..type = refer('BaseUrlManager', 'package:dartrofit/dartrofit.dart')))
      ..body = Code(_bodyCode(library)));
    final generateLibrary = Library(
        (b) => b..body.addAll([baseUrlTypeCls, webEnvCls, instanceMethod]));
    return DartFormatter()
        .format('${generateLibrary.accept(DartEmitter.scoped())}');
  }

  String _bodyCode(LibraryReader library) {
    final configStrList = config.mapToList((type, v) {
      final envUrl = v as Map;

      final envUrlStrList = envUrl.mapToList((env, url) {
        return "envConfig.create(WebEnv.$env, '$url');";
      }).join('\n');

      return '''
      typeConfig.create(BaseUrlType.$type, (envConfig) {
        $envUrlStrList
      });
      ''';
    }).join('\n');
    return '''
    mgr.configure((typeConfig) {
      $configStrList
    });    
    ''';
  }
}
