import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/visitor.dart';
import 'package:code_builder/code_builder.dart';
import 'package:dartrofit/dartrofit.dart';
import 'package:dartrofit_generator/src/checkers.dart';
import 'package:dartrofit_generator/src/element_annotations.dart';

class ParameterVisitor extends SimpleElementVisitor {
  Set<Parameter> requiredParameters;
  Set<Parameter> optionalParameters;

  List<AnnotationInfo> queryInfos = <AnnotationInfo>[];
  List<AnnotationInfo> queryMapInfos = <AnnotationInfo>[];
  AnnotationInfo bodyInfo;
  AnnotationInfo headerInfo;
  AnnotationInfo headerMapInfo;
  AnnotationInfo partFieldInfo;
  AnnotationInfo partFieldMapInfo;
  AnnotationInfo partFileInfo;
  AnnotationInfo partFileListInfo;

  ParameterVisitor();

  @override
  Object visitParameterElement(ParameterElement element) {
    var parameter = Parameter((ParameterBuilder b) => b
      ..type = refer(element.type.getDisplayString())
      ..annotations.addAll([])
      ..name = element.name);
    if (element.isNotOptional) {
      requiredParameters.add(parameter);
    } else if (element.isOptional) {
      if (element.isOptionalNamed) {
        final builder = parameter.toBuilder();
        builder.named = true;
        parameter = builder.build();
      }
      optionalParameters.add(parameter);
    }

    for (var annotation in element.metadata) {
      if (ElementAnnotations.isDartrofitQuery(annotation)) {
        final name = annotation
            .computeConstantValue()
            ?.getField('name')
            ?.toStringValue();

        final queryInfo = AnnotationInfo(element.name, "QueryHandler('$name')");
        queryInfos.add(queryInfo);
      } else if (ElementAnnotations.isDartrofitQueryMap(annotation)) {
        checkQueryMap(element);

        final queryMapInfo = AnnotationInfo(element.name, 'QueryMapHandler()');
        queryMapInfos.add(queryMapInfo);
      } else if (ElementAnnotations.isDartrofitBody(annotation)) {
        checkRequestBody(element);

        final encodeName = annotation
            .computeConstantValue()
            ?.getField('encodeName')
            ?.toStringValue();
        checkEncodingName(element, encodeName);

        bodyInfo = AnnotationInfo(element.name, "BodyHandler('$encodeName')");
      } else if (ElementAnnotations.isDartrofitHeader(annotation)) {
        final name = annotation
            .computeConstantValue()
            ?.getField('name')
            ?.toStringValue();
        headerInfo = AnnotationInfo(element.name, "HeaderHandler('$name')");
      } else if (ElementAnnotations.isDartrofitHeaderMap(annotation)) {
        headerMapInfo = AnnotationInfo(element.name, 'HeaderMapHandler()');
      } else if (ElementAnnotations.isDartrofitPartField(annotation)) {
        final name = annotation
            .computeConstantValue()
            ?.getField('name')
            ?.toStringValue();
        partFieldInfo =
            AnnotationInfo(element.name, "PartFieldHandler('$name')");
      } else if (ElementAnnotations.isDartrofitPartFieldMap(annotation)) {
        partFieldMapInfo =
            AnnotationInfo(element.name, 'PartFieldMapHandler()');
      } else if (ElementAnnotations.isDartrofitPartFile(annotation)) {
        partFileInfo = AnnotationInfo(element.name, 'PartFileHandler()');
      } else if (ElementAnnotations.isDartrofitPartFileList(annotation)) {
        partFileListInfo =
            AnnotationInfo(element.name, 'PartFileListHandler()');
      }
    }
    return super.visitParameterElement(element);
  }
}

class AnnotationInfo {
  final String methodParameterName;
  final String parameterHandlerInstanceStr;

  AnnotationInfo(this.methodParameterName, this.parameterHandlerInstanceStr)
      : assert(methodParameterName != null),
        assert(parameterHandlerInstanceStr != null);
}
